/*********************************************************************/
/* tclparse V2.00 from 14.March 1997                                 */
/* Copyright 1996/97 Stephan Weiss, Heiko Grossmann                  */
/*                   University of Stuttgart, Germany                */
/* For details see the copyright file included in the distribution.  */
/* This notice must not be removed from the file.                    */
/*                                                                   */
/* routines needed for file actions                                  */
/*                                                                   */
/*********************************************************************/

#include "../include/file.h"
#include "../include/stack.h"
#include "../include/const.h"
#include "../include/display.h"


CInputFile::CInputFile( void )
{
  strcpy( buffer, "\0" );
  strcpy( atom.next, "\0" );
  strcpy( atom.actual, "\0" );
  strcpy( atom.last, "\0" );
  strcpy( atom.nextID, "\0" );
  strcpy( atom.actualID, "\0" );
  strcpy( atom.lastID, "\0" );
  strcpy( symbol.next, "\0" );
  strcpy( symbol.actual, "\0" );
  strcpy( symbol.last, "\0" );
  strcpy( symbol.nextID, "\0" );
  strcpy( symbol.actualID, "\0" );
  strcpy( symbol.lastID, "\0" );
  strcpy( word.actual, "\0" );
  strcpy( word.last, "\0" );
  strcpy( word.actualID, "\0" );
  strcpy( word.lastID, "\0" );
  strcpy( unit.actual, "\0" );
  strcpy( unit.last, "\0" );
  strcpy( unit.actualID, "\0" );
  strcpy( unit.lastID, "\0" );

  cursor.x = 1;
  cursor.y = 1;

  name = new char[ 1 ];
  name[ 0 ] = '\0';

  for( int i=0 ; i < 4 ; i++ )
    errorOffset[ i ] = 0;

  size = 0;
}


CInputFile::CInputFile( char *fileName )
{
  strcpy( buffer, "\0" );
  strcpy( atom.next, "\0" );
  strcpy( atom.actual, "\0" );
  strcpy( atom.last, "\0" );
  strcpy( atom.nextID, "\0" );
  strcpy( atom.actualID, "\0" );
  strcpy( atom.lastID, "\0" );
  strcpy( symbol.next, "\0" );
  strcpy( symbol.actual, "\0" );
  strcpy( symbol.last, "\0" );
  strcpy( symbol.nextID, "\0" );
  strcpy( symbol.actualID, "\0" );
  strcpy( symbol.lastID, "\0" );
  strcpy( word.actual, "\0" );
  strcpy( word.last, "\0" );
  strcpy( word.actualID, "\0" );
  strcpy( word.lastID, "\0" );
  strcpy( unit.actual, "\0" );
  strcpy( unit.last, "\0" );
  strcpy( unit.actualID, "\0" );
  strcpy( unit.lastID, "\0" );

  cursor.x = 1;
  cursor.y = 1;

  name = new char[ strlen( fileName )+1 ];
  strcpy( name, fileName );

  for( int i=0 ; i < 4 ; i++ )
    errorOffset[ i ] = 0;

  size = 0;
}


CInputFile::~CInputFile( void )
{
}


void CInputFile::SetErrorOffset( void )
{
  errorOffset[ 0 ] = errorOffset[ 1 ];
  errorOffset[ 1 ] = errorOffset[ 2 ];
  errorOffset[ 2 ] = errorOffset[ 3 ];
  errorOffset[ 3 ] = Tell();
}


streampos CInputFile::GetErrorOffset( void )
{
  return errorOffset[ 0 ];
}


char *CInputFile::Buffer( void )
{
  return &buffer[0];
}


int CInputFile::Open( void )
{
  return Open( name );
}


int CInputFile::Open( char *fileName )
{
  SetName( fileName );
  file.open( fileName );
  if( file.fail() )
    {
      display.Message( "error opening file '", fileName, "'" );
      return 0;
    }

  else
    {
      //Den Dateizeiger an das Ende der Datei stellen, um so die
      //Dateigroesse ermitteln zu koennen.
      file.seekg( 0, ios::end );
      size = file.tellg();
      file.seekg( 0 );

      return 1;
    }
}


void CInputFile::CopyToBuffer( char *string )
{
  strcpy( buffer, string );
}


void CInputFile::Close( void )
{
  file.close();
}


char *CInputFile::GetChar( void )
{
  file.read( buffer, 1 );
  buffer[1] = '\0';
  return Buffer();
}


char *CInputFile::GetLine( void )
{
  file.getline( buffer, unitLength, '\n' );
  return Buffer();
}


int CInputFile::Eof( void )
{
  return file.eof();
}


char *CInputFile::Name( void )
{
  strcpy( buffer, name );
  return Buffer();
}


void CInputFile::Seek( streampos offset )
{
  file.seekg( offset );
}


void CInputFile::SetName( char *fileName )
{
  strcpy( name, fileName );
}


int CInputFile::Size( void )
{
  return size;
}


streampos CInputFile::Tell( void )
{
  return file.tellg();
}


void CInputFile::EnableBraceFlag( void )
{
  braceFlag = 1;
}


void CInputFile::EnableScanFlag( void )
{
  scanFlag = 1;
}


void CInputFile::DisableBraceFlag( void )
{
  braceFlag = 0;
}


void CInputFile::DisableScanFlag( void )
{
  scanFlag = 0;
}


void CInputFile::Save( void )
{
  stack.Push( normal, buffer );
  stack.Push( normal, atom.last );
  stack.Push( normal, atom.actual );
  stack.Push( normal, atom.next );
  stack.Push( normal, atom.lastID );
  stack.Push( normal, atom.actualID );
  stack.Push( normal, atom.nextID );
  stack.Push( normal, symbol.last );
  stack.Push( normal, symbol.actual );
  stack.Push( normal, symbol.next );
  stack.Push( normal, symbol.lastID );
  stack.Push( normal, symbol.actualID );
  stack.Push( normal, symbol.nextID );
  stack.Push( normal, word.last );
  stack.Push( normal, word.actual );
  stack.Push( normal, word.lastID );
  stack.Push( normal, word.actualID );
  stack.Push( normal, unit.last );
  stack.Push( normal, unit.actual );
  stack.Push( normal, unit.lastID );
  stack.Push( normal, unit.actualID );

  save.saveStack = stack;

  save.cursorx = cursor.x;
  save.cursory = cursor.y;
  save.errorx = error.x;
  save.errory = error.y;
  save.braceFlag = braceFlag;
  save.scanFlag = scanFlag;

  for( int i=0 ; i < 4 ; i++ )
    save.saveErrorOffset[ i ] = errorOffset[ i ];

  save.offset = Tell();
}


void CInputFile::Restore( void )
{
  char *dummy;
  stack = save.saveStack;

  strcpy( unit.actualID, stack.Pop( normal ) );
  strcpy( unit.lastID, stack.Pop( normal ) );
  strcpy( unit.actual, stack.Pop( normal ) );
  strcpy( unit.last, stack.Pop( normal ) );
  strcpy( word.actualID, stack.Pop( normal ) );
  strcpy( word.lastID, stack.Pop( normal ) );
  strcpy( word.actual, stack.Pop( normal ) );
  strcpy( word.last, stack.Pop( normal ) );
  strcpy( symbol.nextID, stack.Pop( normal ) );
  strcpy( symbol.actualID, stack.Pop( normal ) );
  strcpy( symbol.lastID, stack.Pop( normal ) );
  strcpy( symbol.next, stack.Pop( normal ) );
  strcpy( symbol.actual, stack.Pop( normal ) );
  strcpy( symbol.last, stack.Pop( normal ) );
  strcpy( atom.nextID, stack.Pop( normal ) );
  strcpy( atom.actualID, stack.Pop( normal ) );
  strcpy( atom.lastID, stack.Pop( normal ) );
  strcpy( atom.next, stack.Pop( normal ) );
  strcpy( atom.actual, stack.Pop( normal ) );
  strcpy( atom.last, stack.Pop( normal ) );
  strcpy( buffer, stack.Pop( normal ) );

  cursor.x = save.cursorx;
  cursor.y = save.cursory;
  error.x = save.errorx;
  error.y = save.errory;
  braceFlag = save.braceFlag;
  scanFlag = save.scanFlag;

  for( int i=0 ; i < 4 ; i++ )
    errorOffset[ i ] = save.saveErrorOffset[ i ];

  Seek( save.offset );
}


void CInputFile::DumpStacks( void )
{
  cout << "\n";
  cout << "parenStack     : " << stack.Concat( parens, " - " ) << '\n'; 
  cout << "braceStack     : " << stack.Concat( braces, " - " ) << '\n'; 
  cout << "quoteStack     : " << stack.Concat( quotes, " - " ) << '\n'; 
  cout << "bracketStack   : " << stack.Concat( brackets, " - " ) << '\n'; 
  cout << "procedureStack : " << stack.Concat( procedures, " - " ) << '\n'; 
  cout << "commandStack   : " << stack.Concat( commands, " - " ) << '\n'; 
  cout << "normalStack    : " << stack.Concat( normal, " - " ) << "\n\n"; 
}


CInputFile *actualFile;
